var moment = require("moment");

module.exports = function (grunt) {

	var backendModules = ['../../teknisa/cnc'];
	var directories = ['utilities', 'repositories', 'services', 'controllers'];
	var subModules = [];
	var libFiles = [
		'zh-grid-navigation/src/GridNavigationLibrary.js',
        'zh-interval-component/dist/zh-interval-component.js',

        'zh-grid-configuration/dist/zh-grid-configuration.js',
        'zh-preferences/dist/preferences.js',
        'zh-filter-preferences/dist/filter-preferences.js',
        'zh-layout-preferences/dist/layout-preferences.js',
        'zh-customization-preferences/dist/view-preferences.js',
		'zh-customization-preferences/dist/zhCustomizationsPreferences.js',

		'zh-crosstab/dist/zh-crosstab.js',
        'zh-cache-bypass/src/CacheBypass.js',
        'zh-reports/dist/zhReports.js',
        'zh-select-autocomplete/dist/zh-select-autocomplete.js',
        'zhuaclfrontend/assets/dist/main.js',
        'zhulog/dist/main.js',
        'tek-libraries/dist/main.js',
		'zh-charts/dist/zh-charts.js',
		'zh-image-uploader/dist/zh-image-uploader.js'
	];

	var cssDependencies = [
		'./bower_components/zeedhi-frontend/assets/css/libs/bootstrap.css',
        './bower_components/zh-grid-navigation/src/style.css',
        './bower_components/zh-charts/dist/zh-charts.css',
		'./bower_components/zh-customizations-service/src/style/style.css',
        './bower_components/zh-select-autocomplete/src/styles/zh-select-autocomplete.css',
        './bower_components/zh-select-autocomplete/dist/style/colors/styles-inflightor.css',
		'./bower_components/zh-image-uploader/dist/zh-image-uploader.css',
		'./assets/css/style.css',
		'./bower_components/zh-reports/dist/style/colors/styles-inflightor.css',
		'./bower_components/zh-preferences/dist/style/style.css',
		'./bower_components/zh-preferences/dist/style/colors/styles-inflightor.css',
		'./bower_components/zh-grid-configuration/dist/style/zh-grid-configuration.css',
		'./bower_components/zh-grid-configuration/dist/style/colors/styles-inflightor.css'
	];
	
	grunt.initConfig({
		concat: {
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSPaths(),
				dest: './assets/js/appes6.js'
			},
			css: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: getCSSPaths(),
				dest: './assets/css/app.css'
			},
			buildLibs: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getLibPaths(),
				dest: './assets/js/libs.js'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'./assets/js/app.min.js': getJSPaths(),
					'./assets/js/libs.min.js': ['./assets/js/libs.js']
				}
			}
		},
		jshint: {
			all: getJSPaths().concat(["GruntFile.js"]),
			options: {
				evil: true
			}
		},
		watch: {
			pivotal: {
				files: getJSPaths().concat(["GruntFile.js"]),
				tasks: [ /*'jshint',*/ 'concat:js', 'concat:css']
			}
		},
		cssmin: {
			dist: {
				src: getCSSPaths(),
				dest: './assets/css/app.min.css'
			}
		},
        babel: {
            options: {
                compact: false,
                sourceMap: true,
                presets: ['env']
            },
            dist: {
                files: {
                    'assets/js/app.js': 'assets/js/appes6.js'
                }
            }
        },
		zhWords: {
			options: {
				serverUrl: "https://translate.teknisa.com/backend/index.php",
				wordsPath: "./assets/json/words",
				organization: "20",
				project: "8987",
		        repository: "1564",
				birtBasePath: "../../teknisa/cnc"
			}
		},
		zhIdGenerator: {								
			options: {									
				jsonPath: getJSONPaths(),				
				menuPath: ["./assets/json/menu.json"]   
			}
		},
		buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
        buildEnvironment: {
			development: {
				configFile: "./config/devEnvironment.json"
			},
			production: {
				configFile: "./config/prodEnvironment.json"
			},
			test: {
				configFile: "./config/testEnvironment.json"
			},
			srcFolders: ["../backend/config/", "./", 
						 "./assets", "./assets/json", 
						 "../../teknisa/cnc/reports", 
						 "../../teknisa/cnc/reports/config", 
						 "../../teknisa/cnc/src/Teknisa/CNC/Utilities"]
		}
	});

	function getJSPaths() {
		var paths = [];
		directories.forEach(function (directory) {
			paths.push('./assets/js/' + directory + '/**/*.js');
			subModules.forEach(function (subModule) {
				paths.push('./assets/modules/' + subModule + '/assets/js/' + directory + '/**/*.js');
			});
		});
		return paths;
	}
	
	function getCSSPaths() {
		var paths = [];
		cssDependencies.forEach(dependency => {
			paths.push(dependency);
			subModules.forEach(function (subModule) {
				paths.push('./assets/modules/' + subModule + '/assets/css/**/*.css');
				paths.push('!./assets/modules/' + subModule + '/assets/css/app.css');
			});
		});
		
		return paths;
	}

	function getLibPaths() {
		var paths = [];
		libFiles.forEach(function(fileName) {
			paths.push('bower_components/' + fileName);
		});
		return paths;
	}

	function getJSONPaths() {
		var paths = ["./assets/json"];
		subModules.forEach(function (subModule) {
			paths.push("./assets/modules/" + subModule + "/assets/json");
		});
		return paths;
	}

	function buildFileContent(srcFiles, filePath, directory) {
		var parts = filePath.split('/');
		parts.shift();
		parts.shift();
		var filename = parts[parts.length - 1];
		var extension = filename.split('.').pop().toUpperCase();
		if (extension == 'JSON' || extension == 'XML') {
			if (directory == 'datasources') {
				filename = parts.pop();
				filename = filename.replace(/\.[^/.]+$/, "");

				if(parts[0] == 'vendor') {
					srcFiles[directory][filename] = './../../backend/' + parts.join('/');
				} else {
					srcFiles[directory][filename] = './../../../' + parts.join('/');
				}
			} else {
				if(parts[0] == 'vendor') {
					srcFiles[directory].push('/../backend/' + parts.join('/'));
				} else {
					srcFiles[directory].push('/../../' + parts.join('/'));
				}
			}
		}
	}
	
	function updateIndexVersion() {
		var indexTemplatePath = './index.html';
		var indexPath = './index.html';
		var indexContent = grunt.file.read(indexTemplatePath);

		indexContent = indexContent.replace(/APPLICATION_VERSION/gm, moment().valueOf() + "");

		grunt.file.write(indexPath, indexContent);
	}

	function updateVersion() {
		updateIndexVersion();
	}

	grunt.registerTask('updateVersion', 'Update version on index.html to not cache project files.', updateVersion);

	grunt.registerTask('backendStep', 'build backend config files', function () {
		var srcFiles = {
			config: [],
			datasources: {}
		};
		var directories = ['config', 'datasources'];
		var dest = {
			config: '../backend/config/servicesMapping.json',
			datasources: '../backend/config/dataSourcesMapping.json'
		};
		
		directories.forEach(function (directory) {
			backendModules.forEach(function (module) {
				var moduleParts = module.split('/');
				var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
				var subModules = grunt.file.expand({
					filter: 'isDirectory'
				}, module + '/src/Teknisa/' + moduleName + '/Modules/*');

				subModules.forEach(function (subModule) {
					grunt.file.expand({
						filter: 'isFile'
					}, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
						buildFileContent(srcFiles, filePath, directory);
					});
				});

				grunt.file.expand({
					filter: 'isFile'
				}, module + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
			});
		});
		grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
	});

	grunt.loadNpmTasks('grunt-babel');
	
	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-cssmin');
	grunt.loadNpmTasks('grunt-zh-id-generator');
	grunt.loadNpmTasks('grunt-zhuacl-structure');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-words');
	
	grunt.registerTask('default', [ /*'jshint',*/ 'zhWords', 'backendStep', 'concat', 'cssmin', 'buildEnvironment', 'babel', 'buildAclStruct', 'updateVersion']);
	grunt.registerTask('buildLibs', ['concat:buildLibs']);
	grunt.registerTask('dist', [ /*'jshint',*/ 'backendStep', 'uglify', 'cssmin']);
};

